module Btrb

module Bencoding

  module Decode
    def self.new_object(b)
      case b
      when ?d
        return Dictionay.new()
      when ?i
        return Int.new()
      when ?l
        return List.new()
      else
        if '0123456789'.include?(b)
          return Bstr.new(b)
        else
          raise BencodingError, "illegal object start:#{b.chr}"
        end
      end
    end
  end #module Decode

  class BencodingError < RuntimeError
  end

  class Bstr
    def initialize(b)
      @len = b - 0x30
      @value = ''
      @str = nil
    end
    def add(b)
      if @str.nil?
        if b == ?:
          @str = ''
        elsif b < 0x30 || b > 0x39
          raise BencodingError, "illegal char in length-field:#{b.chr}"
        else
          @len *= 10
          @len += b - 0x30
        end
      else
        if @len > 0
          @len -= 1
          @value << b.chr
        end
      end
      return nil if @len == 0
      self
    end
    attr_reader :value
  end

  class Int
    def initialize()
      @value = ''
    end
    def add(b)
      if b == ?e
        @value = @value.to_i
        return nil
      else
        @value << b.chr
      end
      self
    end
    attr_reader :value
  end

  class List
    def initialize()
      @current = nil
      @value = []
    end
    def add(b)
      if @current.nil?
        return nil if b == ?e
        begin
          @current = Decode::new_object(b)
        rescue BencodingError => e
          if $DEBUG
            p @value
            p @current
          end
          raise e
        end
      else
        c = @current.add(b)
        if c.nil?
          @value << @current.value
          @current = nil
          return self
        end
      end
      @current
    end
    attr_reader :value
  end

  class Dictionay
    def initialize()
      @current = nil
      @key = nil
      @value = {}
    end
    def add(b)
      if @current.nil?
        return nil if b == ?e
        begin
          @current = Decode::new_object(b)
        rescue BencodingError => e
          if $DEBUG
            p @value
            p @key
            p @current
          end
          raise e
        end
      else
        c = @current.add(b)
        if c.nil?
          if @key.nil?
            @key = @current.value
          else
            @value[@key] = @current.value
            @key = nil
          end
          @current = nil
          return self
        end
      end
      @current
    end
    attr_reader :value
  end

  def decode(s)
    r = []
    return r if s.nil? || s.size == 0
    c = Decode::new_object(s[0])
    1.upto(s.size - 1) do |i|
      if c.nil?
        c = Decode::new_object(s[i])
      else
        rc = c.add(s[i])
        if rc.nil?
          r << c.value
          c = nil
        end
      end
    end
    r << c.value if !c.nil?
    r
  end
end #module Bencoding

end #moduel Btrb
